=begin
  gettext/erb.rb - Class-based container module for GetText 

  Copyright (C) 2005,2006  Masao Mutoh

  You may redistribute it and/or modify it under the same
  license terms as Ruby.

  $Id: container.rb,v 1.4 2006/02/23 14:53:59 mutoh Exp $
=end

require 'gettext'

module GetText
  # This module is used instead of GetText module. This module provides same functions as GetText module.
  #
  # Normaly, GetText.bindtexdomain should be called once a ruby-script-file.
  # But if the class includes GetText::Container, a TextDomain is bound to an instance of the class. 
  # So you don't need to call bindtextdomain in each files if the files define a class.
  # 
  # This module is useful for application-side classes, but you shouldn't use this for libraries. 
  # Because the user of your library may override/add new features to the original library and the 
  # user needs to prepare another TextDomain(Otherwise, apply po/mo patch to the original library).
  module Container
    include GetText

    alias :_bindtextdomain :bindtextdomain #:nodoc:

    def bindtextdomain(domainname, path = nil, locale = nil, charset = nil) #:nodoc:
      @gettext_container_domainname = domainname
      _bindtextdomain(domainname, path, locale, charset)
    end

    def callersrc #:nodoc:
      @gettext_container_domainname
    end

  end
end
